@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 :mod:`neo.core` provides classes for storing common electrophysiological data
 types.  Some of these classes contain raw data, such as spike trains or
@@ -12,22 +11,30 @@ Classes:
 
 .. autoclass:: Block
 .. autoclass:: Segment
-.. autoclass:: ChannelIndex
-.. autoclass:: Unit
+.. autoclass:: Group
 
 .. autoclass:: AnalogSignal
 .. autoclass:: IrregularlySampledSignal
 
+.. autoclass:: ChannelView
+
 .. autoclass:: Event
 .. autoclass:: Epoch
 
 .. autoclass:: SpikeTrain
+.. autoclass:: ImageSequence
+
+.. autoclass:: RectangularRegionOfInterest
+.. autoclass:: CircularRegionOfInterest
+.. autoclass:: PolygonRegionOfInterest
+
+Deprecated classes:
+
+.. autoclass:: ChannelIndex
+.. autoclass:: Unit
 
 """
 
-# needed for python 3 compatibility
-from __future__ import absolute_import, division, print_function
-
 from neo.core.block import Block
 from neo.core.segment import Segment
 from neo.core.channelindex import ChannelIndex
@@ -41,10 +48,18 @@ from neo.core.epoch import Epoch
 
 from neo.core.spiketrain import SpikeTrain
 
+from neo.core.imagesequence import ImageSequence
+from neo.core.regionofinterest import RectangularRegionOfInterest, CircularRegionOfInterest, PolygonRegionOfInterest
+
+from neo.core.view import ChannelView
+from neo.core.group import Group
+
 # Block should always be first in this list
 objectlist = [Block, Segment, ChannelIndex,
               AnalogSignal, IrregularlySampledSignal,
-              Event, Epoch, Unit, SpikeTrain]
+              Event, Epoch, Unit, SpikeTrain, ImageSequence,
+              RectangularRegionOfInterest, CircularRegionOfInterest,
+              PolygonRegionOfInterest, ChannelView, Group]
 
 objectnames = [ob.__name__ for ob in objectlist]
 class_by_name = dict(zip(objectnames, objectlist))
