@@ -1,12 +1,9 @@
-# -*- coding: utf-8 -*-
 """
 This module defines :class:`BaseNeo`, the abstract base class
 used by all :module:`neo.core` classes.
 """
 
-# needed for python 3 compatibility
-from __future__ import absolute_import, division, print_function
-
+from copy import deepcopy
 from datetime import datetime, date, time, timedelta
 from decimal import Decimal
 import logging
@@ -27,11 +24,6 @@ try:
 except NameError:
     pass
 
-try:
-    basestring
-except NameError:
-    basestring = str
-
 logger = logging.getLogger("Neo")
 
 
@@ -72,7 +64,7 @@ def merge_annotation(a, b):
         For strings: concatenate with ';'
         Otherwise: fail if the annotations are not equal
     """
-    assert type(a) == type(b), 'type(%s) %s != type(%s) %s' % (a, type(a),
+    assert type(a) == type(b), 'type({}) {} != type({}) {}'.format(a, type(a),
                                                                b, type(b))
     if isinstance(a, dict):
         return merge_annotations(a, b)
@@ -80,17 +72,17 @@ def merge_annotation(a, b):
         return np.append(a, b)
     elif isinstance(a, list):  # concatenate b to a
         return a + b
-    elif isinstance(a, basestring):
+    elif isinstance(a, str):
         if a == b:
             return a
         else:
             return a + ";" + b
     else:
-        assert a == b, '%s != %s' % (a, b)
+        assert a == b, '{} != {}'.format(a, b)
         return a
 
 
-def merge_annotations(A, B):
+def merge_annotations(A, *Bs):
     """
     Merge two sets of annotations.
 
@@ -102,24 +94,53 @@ def merge_annotations(A, B):
         For strings: concatenate with ';'
         Otherwise: warn if the annotations are not equal
     """
-    merged = {}
-    for name in A:
-        if name in B:
-            try:
-                merged[name] = merge_annotation(A[name], B[name])
-            except BaseException as exc:
-                # exc.args += ('key %s' % name,)
-                # raise
-                merged[name] = "MERGE CONFLICT"  # temporary hack
-        else:
-            merged[name] = A[name]
-    for name in B:
-        if name not in merged:
-            merged[name] = B[name]
-    logger.debug("Merging annotations: A=%s B=%s merged=%s", A, B, merged)
+    merged = A.copy()
+    for B in Bs:
+        for name in B:
+            if name not in merged:
+                merged[name] = B[name]
+            else:
+                try:
+                    merged[name] = merge_annotation(merged[name], B[name])
+                except BaseException as exc:
+                    # exc.args += ('key %s' % name,)
+                    # raise
+                    merged[name] = "MERGE CONFLICT"  # temporary hack
+    logger.debug("Merging annotations: A=%s Bs=%s merged=%s", A, Bs, merged)
     return merged
 
 
+def intersect_annotations(A, B):
+    """
+    Identify common entries in dictionaries A and B
+    and return these in a separate dictionary.
+
+    Entries have to share key as well as value to be
+    considered common.
+
+    Parameters
+    ----------
+    A, B : dict
+        Dictionaries to merge.
+    """
+
+    result = {}
+
+    for key in set(A.keys()) & set(B.keys()):
+        v1, v2 = A[key], B[key]
+        assert type(v1) == type(v2), 'type({}) {} != type({}) {}'.format(v1, type(v1),
+                                                                         v2, type(v2))
+        if isinstance(v1, dict) and v1 == v2:
+            result[key] = deepcopy(v1)
+        elif isinstance(v1, str) and v1 == v2:
+            result[key] = A[key]
+        elif isinstance(v1, list) and v1 == v2:
+            result[key] = deepcopy(v1)
+        elif isinstance(v1, np.ndarray) and all(v1 == v2):
+            result[key] = deepcopy(v1)
+    return result
+
+
 def _reference_name(class_name):
     """
     Given the name of a class, return an attribute name to be used for
@@ -150,7 +171,7 @@ def _container_name(class_name):
     return name_map.get(class_name, _reference_name(class_name) + 's')
 
 
-class BaseNeo(object):
+class BaseNeo:
     """
     This is the base class from which all Neo objects inherit.
 
@@ -244,10 +265,12 @@ class BaseNeo(object):
     # specified in each child class
     # Parent objects whose children can have a single parent
     _single_parent_objects = ()
+    # Attribute names corresponding to _single_parent_objects
+    _single_parent_attrs = ()
     # Parent objects whose children can have multiple parents
     _multi_parent_objects = ()
 
-    # Attributes that an instance is requires to have defined
+    # Attributes that an instance is required to have defined
     _necessary_attrs = ()
     # Attributes that an instance may or may have defined
     _recommended_attrs = (('name', str),
@@ -306,7 +329,7 @@ class BaseNeo(object):
                 else:
                     pp.breakable()
                 with pp.group(indent=1):
-                    pp.text("{0}: ".format(key))
+                    pp.text("{}: ".format(key))
                     pp.pretty(value)
 
     def _repr_pretty_(self, pp, cycle):
@@ -367,7 +390,7 @@ class BaseNeo(object):
         """
         return self._necessary_attrs + self._recommended_attrs
 
-    def merge_annotations(self, other):
+    def merge_annotations(self, *others):
         """
         Merge annotations from the other object into this one.
 
@@ -379,14 +402,27 @@ class BaseNeo(object):
             For strings: concatenate with ';'
             Otherwise: fail if the annotations are not equal
         """
+        other_annotations = [other.annotations for other in others]
         merged_annotations = merge_annotations(self.annotations,
-                                               other.annotations)
+                                               *other_annotations)
         self.annotations.update(merged_annotations)
 
-    def merge(self, other):
+    def merge(self, *others):
         """
         Merge the contents of another object into this one.
 
         See :meth:`merge_annotations` for details of the merge operation.
         """
-        self.merge_annotations(other)
+        self.merge_annotations(*others)
+
+    def set_parent(self, obj):
+        """
+        Set the appropriate "parent" attribute of this object
+        according to the type of "obj"
+        """
+        if obj.__class__.__name__ not in self._single_parent_objects:
+            raise TypeError("{} can only have parents of type {}, not {}".format(
+                self.__class__.__name__, self._single_parent_objects, obj.__class__.__name__))
+        loc = self._single_parent_objects.index(obj.__class__.__name__)
+        parent_attr = self._single_parent_attrs[loc]
+        setattr(self, parent_attr, obj)
