@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 '''
 This module defines :class:`Event`, an array of events.
 
@@ -6,11 +5,7 @@ This module defines :class:`Event`, an array of events.
 :module:`neo.core.baseneo`.
 '''
 
-# needed for python 3 compatibility
-from __future__ import absolute_import, division, print_function
-
-import sys
-from copy import deepcopy
+from copy import deepcopy, copy
 
 import numpy as np
 import quantities as pq
@@ -19,8 +14,6 @@ from neo.core.baseneo import merge_annotations
 from neo.core.dataobject import DataObject, ArrayDict
 from neo.core.epoch import Epoch
 
-PY_VER = sys.version_info[0]
-
 
 def _new_event(cls, times=None, labels=None, units=None, name=None, file_origin=None,
                description=None, array_annotations=None, annotations=None, segment=None):
@@ -46,17 +39,17 @@ class Event(DataObject):
         >>>
         >>> evt = Event(np.arange(0, 30, 10)*s,
         ...             labels=np.array(['trig0', 'trig1', 'trig2'],
-        ...                             dtype='S'))
+        ...                             dtype='U'))
         >>>
         >>> evt.times
         array([  0.,  10.,  20.]) * s
         >>> evt.labels
         array(['trig0', 'trig1', 'trig2'],
-              dtype='|S5')
+              dtype='<U5')
 
     *Required attributes/properties*:
         :times: (quantity array 1D) The time of the events.
-        :labels: (numpy.array 1D dtype='S') Names or labels for the events.
+        :labels: (numpy.array 1D dtype='U' or list) Names or labels for the events.
 
     *Recommended attributes/properties*:
         :name: (str) A label for the dataset.
@@ -73,15 +66,24 @@ class Event(DataObject):
     '''
 
     _single_parent_objects = ('Segment',)
+    _single_parent_attrs = ('segment',)
     _quantity_attr = 'times'
-    _necessary_attrs = (('times', pq.Quantity, 1), ('labels', np.ndarray, 1, np.dtype('S')))
+    _necessary_attrs = (('times', pq.Quantity, 1), ('labels', np.ndarray, 1, np.dtype('U')))
 
     def __new__(cls, times=None, labels=None, units=None, name=None, description=None,
                 file_origin=None, array_annotations=None, **annotations):
         if times is None:
             times = np.array([]) * pq.s
+        elif isinstance(times, (list, tuple)):
+            times = np.array(times)
+        if len(times.shape) > 1:
+            raise ValueError("Times array has more than 1 dimension")
         if labels is None:
-            labels = np.array([], dtype='S')
+            labels = np.array([], dtype='U')
+        else:
+            labels = np.array(labels)
+            if labels.size != times.size and labels.size:
+                raise ValueError("Labels array has different length to times")
         if units is None:
             # No keyword units, so get from `times`
             try:
@@ -99,10 +101,10 @@ class Event(DataObject):
         # reference dimensionality
         if (len(dim) != 1 or list(dim.values())[0] != 1 or not isinstance(list(dim.keys())[0],
                                                                           pq.UnitTime)):
-            ValueError("Unit %s has dimensions %s, not [time]" % (units, dim.simplified))
+            ValueError("Unit {} has dimensions {}, not [time]".format(units, dim.simplified))
 
         obj = pq.Quantity(times, units=dim).view(cls)
-        obj.labels = labels
+        obj._labels = labels
         obj.segment = None
         return obj
 
@@ -124,7 +126,8 @@ class Event(DataObject):
                             self.annotations, self.segment)
 
     def __array_finalize__(self, obj):
-        super(Event, self).__array_finalize__(obj)
+        super().__array_finalize__(obj)
+        self._labels = getattr(obj, 'labels', None)
         self.annotations = getattr(obj, 'annotations', None)
         self.name = getattr(obj, 'name', None)
         self.file_origin = getattr(obj, 'file_origin', None)
@@ -140,23 +143,32 @@ class Event(DataObject):
         '''
         Returns a string representing the :class:`Event`.
         '''
-        # need to convert labels to unicode for python 3 or repr is messed up
-        if PY_VER == 3:
-            labels = self.labels.astype('U')
-        else:
-            labels = self.labels
-        objs = ['%s@%s' % (label, time) for label, time in zip(labels, self.times)]
+
+        objs = ['%s@%s' % (label, str(time)) for label, time in zip(self.labels, self.times)]
         return '<Event: %s>' % ', '.join(objs)
 
     def _repr_pretty_(self, pp, cycle):
-        super(Event, self)._repr_pretty_(pp, cycle)
+        super()._repr_pretty_(pp, cycle)
 
     def rescale(self, units):
         '''
         Return a copy of the :class:`Event` converted to the specified
         units
+        :return: Copy of self with specified units
         '''
-        obj = super(Event, self).rescale(units)
+        # Use simpler functionality, if nothing will be changed
+        dim = pq.quantity.validate_dimensionality(units)
+        if self.dimensionality == dim:
+            return self.copy()
+
+        # Rescale the object into a new object
+        obj = self.duplicate_with_new_data(
+            times=self.view(pq.Quantity).rescale(dim),
+            labels=self.labels,
+            units=units)
+
+        # Expected behavior is deepcopy, so deepcopying array_annotations
+        obj.array_annotations = deepcopy(self.array_annotations)
         obj.segment = self.segment
         return obj
 
@@ -176,6 +188,7 @@ class Event(DataObject):
         '''
         othertimes = other.times.rescale(self.times.units)
         times = np.hstack([self.times, othertimes]) * self.times.units
+        labels = np.hstack([self.labels, other.labels])
         kwargs = {}
         for name in ("name", "description", "file_origin"):
             attr_self = getattr(self, name)
@@ -183,7 +196,7 @@ class Event(DataObject):
             if attr_self == attr_other:
                 kwargs[name] = attr_self
             else:
-                kwargs[name] = "merge(%s, %s)" % (attr_self, attr_other)
+                kwargs[name] = "merge({}, {})".format(attr_self, attr_other)
 
         print('Event: merge annotations')
         merged_annotations = merge_annotations(self.annotations, other.annotations)
@@ -192,7 +205,7 @@ class Event(DataObject):
 
         kwargs['array_annotations'] = self._merge_array_annotations(other)
 
-        evt = Event(times=times, labels=kwargs['array_annotations']['labels'], **kwargs)
+        evt = Event(times=times, labels=labels, **kwargs)
 
         return evt
 
@@ -201,36 +214,38 @@ class Event(DataObject):
         Copy the metadata from another :class:`Event`.
         Note: Array annotations can not be copied here because length of data can change
         '''
-        # Note: Array annotations cannot be copied
-        # because they are linked to their respective timestamps
-        for attr in ("name", "file_origin", "description", "annotations"):
-            setattr(self, attr, getattr(other, attr,
-                                        None))  # Note: Array annotations cannot be copied
-            # because length of data can be changed  # here which would cause inconsistencies  #
-            #  This includes labels and durations!!!
-
-    def __deepcopy__(self, memo):
-        cls = self.__class__
-        new_ev = cls(times=self.times, labels=self.labels, units=self.units, name=self.name,
-                     description=self.description, file_origin=self.file_origin)
-        new_ev.__dict__.update(self.__dict__)
-        memo[id(self)] = new_ev
-        for k, v in self.__dict__.items():
-            try:
-                setattr(new_ev, k, deepcopy(v, memo))
-            except TypeError:
-                setattr(new_ev, k, v)
-        return new_ev
+        # Note: Array annotations, including labels, cannot be copied
+        # because they are linked to their respective timestamps and length of data can be changed
+        # here which would cause inconsistencies
+        for attr in ("name", "file_origin", "description",
+                     "annotations"):
+            setattr(self, attr, deepcopy(getattr(other, attr, None)))
 
     def __getitem__(self, i):
-        obj = super(Event, self).__getitem__(i)
+        obj = super().__getitem__(i)
+        if self._labels is not None and self._labels.size > 0:
+            obj.labels = self._labels[i]
+        else:
+            obj.labels = self._labels
         try:
             obj.array_annotate(**deepcopy(self.array_annotations_at_index(i)))
+            obj._copy_data_complement(self)
         except AttributeError:  # If Quantity was returned, not Event
-            pass
+            obj.times = obj
         return obj
 
-    def duplicate_with_new_data(self, signal, units=None):
+    def set_labels(self, labels):
+        if self.labels is not None and self.labels.size > 0 and len(labels) != self.size:
+            raise ValueError("Labels array has different length to times ({} != {})"
+                            .format(len(labels), self.size))
+        self._labels = np.array(labels)
+
+    def get_labels(self):
+        return self._labels
+
+    labels = property(get_labels, set_labels)
+
+    def duplicate_with_new_data(self, times, labels, units=None):
         '''
         Create a new :class:`Event` with the same metadata
         but different data
@@ -241,8 +256,9 @@ class Event(DataObject):
         else:
             units = pq.quantity.validate_dimensionality(units)
 
-        new = self.__class__(times=signal, units=units)
+        new = self.__class__(times=times, units=units)
         new._copy_data_complement(self)
+        new.labels = labels
         # Note: Array annotations cannot be copied here, because length of data can be changed
         return new
 
@@ -261,17 +277,35 @@ class Event(DataObject):
             _t_stop = np.inf
 
         indices = (self >= _t_start) & (self <= _t_stop)
-        new_evt = self[indices]
+
+        # Time slicing should create a deep copy of the object
+        new_evt = deepcopy(self[indices])
 
         return new_evt
 
-    def set_labels(self, labels):
-        self.array_annotate(labels=labels)
+    def time_shift(self, t_shift):
+        """
+        Shifts an :class:`Event` by an amount of time.
 
-    def get_labels(self):
-        return self.array_annotations['labels']
+        Parameters:
+        -----------
+        t_shift: Quantity (time)
+            Amount of time by which to shift the :class:`Event`.
 
-    labels = property(get_labels, set_labels)
+        Returns:
+        --------
+        epoch: Event
+            New instance of an :class:`Event` object starting at t_shift later than the
+            original :class:`Event` (the original :class:`Event` is not modified).
+        """
+        new_evt = self.duplicate_with_new_data(times=self.times + t_shift,
+                                               labels=self.labels)
+
+        # Here we can safely copy the array annotations since we know that
+        # the length of the Event does not change.
+        new_evt.array_annotate(**self.array_annotations)
+
+        return new_evt
 
     def to_epoch(self, pairwise=False, durations=None):
         """
