@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 '''
 This module defines :class:`Segment`, a container for data sharing a common
 time basis.
@@ -7,13 +6,12 @@ time basis.
 from :module:`neo.core.container`.
 '''
 
-# needed for python 3 compatibility
-from __future__ import absolute_import, division, print_function
-
 from datetime import datetime
 
 import numpy as np
 
+from copy import deepcopy
+
 from neo.core.container import Container
 
 
@@ -76,7 +74,7 @@ class Segment(Container):
 
     _data_child_objects = ('AnalogSignal',
                            'Epoch', 'Event',
-                           'IrregularlySampledSignal', 'SpikeTrain')
+                           'IrregularlySampledSignal', 'SpikeTrain', 'ImageSequence')
     _single_parent_objects = ('Block',)
     _recommended_attrs = ((('file_datetime', datetime),
                            ('rec_datetime', datetime),
@@ -90,7 +88,7 @@ class Segment(Container):
         '''
         Initialize a new :class:`Segment` instance.
         '''
-        super(Segment, self).__init__(name=name, description=description,
+        super().__init__(name=name, description=description,
                                       file_origin=file_origin, **annotations)
 
         self.file_datetime = file_datetime
@@ -105,7 +103,12 @@ class Segment(Container):
         '''
         t_starts = [sig.t_start for sig in self.analogsignals +
                     self.spiketrains + self.irregularlysampledsignals]
-        t_starts += [e.times[0] for e in self.epochs + self.events if len(e.times) > 0]
+
+        for e in self.epochs + self.events:
+            if hasattr(e, 't_start'):  # in case of proxy objects
+                t_starts += [e.t_start]
+            elif len(e) > 0:
+                t_starts += [e.times[0]]
 
         # t_start is not defined if no children are present
         if len(t_starts) == 0:
@@ -122,7 +125,12 @@ class Segment(Container):
         '''
         t_stops = [sig.t_stop for sig in self.analogsignals +
                    self.spiketrains + self.irregularlysampledsignals]
-        t_stops += [e.times[-1] for e in self.epochs + self.events if len(e.times) > 0]
+
+        for e in self.epochs + self.events:
+            if hasattr(e, 't_stop'):  # in case of proxy objects
+                t_stops += [e.t_stop]
+            elif len(e) > 0:
+                t_stops += [e.times[-1]]
 
         # t_stop is not defined if no children are present
         if len(t_stops) == 0:
@@ -245,9 +253,112 @@ class Segment(Container):
             2
 
         '''
+        # todo: provide equivalent method using Group/ChannelView
+        #       add deprecation message (use decorator)?
         seg = Segment()
         seg.spiketrains = self.take_spiketrains_by_unit(unit_list)
         seg.analogsignals = \
             self.take_slice_of_analogsignalarray_by_unit(unit_list)
         # TODO copy others attributes
         return seg
+
+    def time_slice(self, t_start=None, t_stop=None, reset_time=False, **kwargs):
+        """
+        Creates a time slice of a Segment containing slices of all child
+        objects.
+
+        Parameters:
+        -----------
+        t_start: Quantity
+            Starting time of the sliced time window.
+        t_stop: Quantity
+            Stop time of the sliced time window.
+        reset_time: bool
+            If True the time stamps of all sliced objects are set to fall
+            in the range from t_start to t_stop.
+            If False, original time stamps are retained.
+            Default is False.
+
+        Keyword Arguments:
+        ------------------
+            Additional keyword arguments used for initialization of the sliced
+            Segment object.
+
+        Returns:
+        --------
+        subseg: Segment
+            Temporal slice of the original Segment from t_start to t_stop.
+        """
+        subseg = Segment(**kwargs)
+
+        for attr in ['file_datetime', 'rec_datetime', 'index',
+                     'name', 'description', 'file_origin']:
+            setattr(subseg, attr, getattr(self, attr))
+
+        subseg.annotations = deepcopy(self.annotations)
+
+        if t_start is None:
+            t_start = self.t_start
+        if t_stop is None:
+            t_stop = self.t_stop
+
+        t_shift = - t_start
+
+        # cut analogsignals and analogsignalarrays
+        for ana_id in range(len(self.analogsignals)):
+            if hasattr(self.analogsignals[ana_id], '_rawio'):
+                ana_time_slice = self.analogsignals[ana_id].load(time_slice=(t_start, t_stop))
+            else:
+                ana_time_slice = self.analogsignals[ana_id].time_slice(t_start, t_stop)
+            if reset_time:
+                ana_time_slice = ana_time_slice.time_shift(t_shift)
+            subseg.analogsignals.append(ana_time_slice)
+
+        # cut irregularly sampled signals
+        for irr_id in range(len(self.irregularlysampledsignals)):
+            if hasattr(self.irregularlysampledsignals[irr_id], '_rawio'):
+                ana_time_slice = self.irregularlysampledsignals[irr_id].load(
+                    time_slice=(t_start, t_stop))
+            else:
+                ana_time_slice = self.irregularlysampledsignals[irr_id].time_slice(t_start, t_stop)
+            if reset_time:
+                ana_time_slice = ana_time_slice.time_shift(t_shift)
+            subseg.irregularlysampledsignals.append(ana_time_slice)
+
+        # cut spiketrains
+        for st_id in range(len(self.spiketrains)):
+            if hasattr(self.spiketrains[st_id], '_rawio'):
+                st_time_slice = self.spiketrains[st_id].load(time_slice=(t_start, t_stop))
+            else:
+                st_time_slice = self.spiketrains[st_id].time_slice(t_start, t_stop)
+            if reset_time:
+                st_time_slice = st_time_slice.time_shift(t_shift)
+            subseg.spiketrains.append(st_time_slice)
+
+        # cut events
+        for ev_id in range(len(self.events)):
+            if hasattr(self.events[ev_id], '_rawio'):
+                ev_time_slice = self.events[ev_id].load(time_slice=(t_start, t_stop))
+            else:
+                ev_time_slice = self.events[ev_id].time_slice(t_start, t_stop)
+            if reset_time:
+                ev_time_slice = ev_time_slice.time_shift(t_shift)
+            # appending only non-empty events
+            if len(ev_time_slice):
+                subseg.events.append(ev_time_slice)
+
+        # cut epochs
+        for ep_id in range(len(self.epochs)):
+            if hasattr(self.epochs[ep_id], '_rawio'):
+                ep_time_slice = self.epochs[ep_id].load(time_slice=(t_start, t_stop))
+            else:
+                ep_time_slice = self.epochs[ep_id].time_slice(t_start, t_stop)
+            if reset_time:
+                ep_time_slice = ep_time_slice.time_shift(t_shift)
+            # appending only non-empty epochs
+            if len(ep_time_slice):
+                subseg.epochs.append(ep_time_slice)
+
+        subseg.create_relationship()
+
+        return subseg
