@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 baseio
 ======
@@ -12,21 +11,26 @@ BaseIO        - abstract class which should be overridden, managing how a
 If you want a model for developing a new IO start from exampleIO.
 """
 
-import collections
+try:
+    from collections.abc import Sequence
+except ImportError:
+    from collections import Sequence
 import logging
 
 from neo import logging_handler
 from neo.core import (AnalogSignal, Block,
-                      Epoch, Event,
+                      Epoch, Event, Group,
                       IrregularlySampledSignal,
-                      ChannelIndex,
-                      Segment, SpikeTrain, Unit)
+                      ChannelIndex, ChannelView,
+                      Segment, SpikeTrain, Unit, ImageSequence,
+                      RectangularRegionOfInterest, CircularRegionOfInterest,
+                      PolygonRegionOfInterest)
 
 read_error = "This type is not supported by this file format for reading"
 write_error = "This type is not supported by this file format for writing"
 
 
-class BaseIO(object):
+class BaseIO:
     """
     Generic class to handle all the file read/write methods for the key objects
     of the core class. This template is file-reading/writing oriented but it
@@ -108,8 +112,11 @@ class BaseIO(object):
 
     ######## General read/write methods #######################
     def read(self, lazy=False, **kargs):
-        if lazy:
-            assert self.support_lazy, 'This IO do not support lazy loading'
+        """
+        Return all data from the file as a list of Blocks
+        """
+        if lazy and not self.support_lazy:
+            raise ValueError("This IO module does not support lazy loading")
         if Block in self.readable_objects:
             if (hasattr(self, 'read_all_blocks') and
                     callable(getattr(self, 'read_all_blocks'))):
@@ -126,7 +133,7 @@ class BaseIO(object):
 
     def write(self, bl, **kargs):
         if Block in self.writeable_objects:
-            if isinstance(bl, collections.Sequence):
+            if isinstance(bl, Sequence):
                 assert hasattr(self, 'write_all_blocks'), \
                     '%s does not offer to store a sequence of blocks' % \
                     self.__class__.__name__
@@ -157,18 +164,36 @@ class BaseIO(object):
     def read_analogsignal(self, **kargs):
         assert (AnalogSignal in self.readable_objects), read_error
 
+    def read_imagesequence(self, **kargs):
+        assert (ImageSequence in self.readable_objects), read_error
+
+    def read_rectangularregionofinterest(self, **kargs):
+        assert (RectangularRegionOfInterest in self.readable_objects), read_error
+
+    def read_circularregionofinterest(self, **kargs):
+        assert (CircularRegionOfInterest in self.readable_objects), read_error
+
+    def read_polygonregionofinterest(self, **kargs):
+        assert (PolygonRegionOfInterest in self.readable_objects), read_error
+
     def read_irregularlysampledsignal(self, **kargs):
         assert (IrregularlySampledSignal in self.readable_objects), read_error
 
     def read_channelindex(self, **kargs):
         assert (ChannelIndex in self.readable_objects), read_error
 
+    def read_channelview(self, **kargs):
+        assert (ChannelView in self.readable_objects), read_error
+
     def read_event(self, **kargs):
         assert (Event in self.readable_objects), read_error
 
     def read_epoch(self, **kargs):
         assert (Epoch in self.readable_objects), read_error
 
+    def read_group(self, **kargs):
+        assert (Group in self.readable_objects), read_error
+
     ######## All individual write methods #######################
     def write_block(self, bl, **kargs):
         assert (Block in self.writeable_objects), write_error
@@ -185,14 +210,32 @@ class BaseIO(object):
     def write_analogsignal(self, anasig, **kargs):
         assert (AnalogSignal in self.writeable_objects), write_error
 
+    def write_imagesequence(self, imseq, **kargs):
+        assert (ImageSequence in self.writeable_objects), write_error
+
+    def write_rectangularregionofinterest(self, rectroi, **kargs):
+        assert (RectangularRegionOfInterest in self.writeable_objects), read_error
+
+    def write_circularregionofinterest(self, circroi, **kargs):
+        assert (CircularRegionOfInterest in self.writeable_objects), read_error
+
+    def write_polygonregionofinterest(self, polyroi, **kargs):
+        assert (PolygonRegionOfInterest in self.writeable_objects), read_error
+
     def write_irregularlysampledsignal(self, irsig, **kargs):
         assert (IrregularlySampledSignal in self.writeable_objects), write_error
 
     def write_channelindex(self, chx, **kargs):
         assert (ChannelIndex in self.writeable_objects), write_error
 
+    def write_channelview(self, chv, **kargs):
+        assert (ChannelView in self.writeable_objects), write_error
+
     def write_event(self, ev, **kargs):
         assert (Event in self.writeable_objects), write_error
 
     def write_epoch(self, ep, **kargs):
         assert (Epoch in self.writeable_objects), write_error
+
+    def write_group(self, group, **kargs):
+        assert (Group in self.writeable_objects), write_error
\ No newline at end of file
