@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 '''
 Class for reading from Brainware F32 files
 
@@ -25,9 +24,6 @@ The code is implemented with the permission of Dr. Jan Schnupp
 Author: Todd Jennings
 '''
 
-# needed for python 3 compatibility
-from __future__ import absolute_import, division, print_function
-
 # import needed core python modules
 from os import path
 
@@ -36,7 +32,7 @@ import numpy as np
 import quantities as pq
 
 # needed core neo modules
-from neo.core import Block, ChannelIndex, Segment, SpikeTrain, Unit
+from neo.core import Block, Group, Segment, SpikeTrain, Unit
 
 # need to subclass BaseIO
 from neo.io.baseio import BaseIO
@@ -63,8 +59,7 @@ class BrainwareF32IO(BaseIO):
     reading or closed.
 
     Note 1:
-        There is always only one ChannelIndex.  BrainWare stores the
-        equivalent of ChannelIndexes in separate files.
+        There is always only one Group.
 
     Usage:
         >>> from neo.io.brainwaref32io import BrainwareF32IO
@@ -84,7 +79,7 @@ class BrainwareF32IO(BaseIO):
 
     # This class is able to directly or indirectly handle the following objects
     # You can notice that this greatly simplifies the full Neo object hierarchy
-    supported_objects = [Block, ChannelIndex,
+    supported_objects = [Block, Group,
                          Segment, SpikeTrain, Unit]
 
     readable_objects = [Block]
@@ -121,19 +116,13 @@ class BrainwareF32IO(BaseIO):
         self._fsrc = None
 
         self._blk = None
-        self.__unit = None
+        self.__unit_group = None
 
         self.__t_stop = None
         self.__params = None
         self.__seg = None
         self.__spiketimes = None
 
-    def read(self, lazy=False, **kargs):
-        '''
-        Reads simple spike data file "fname" generated with BrainWare
-        '''
-        return self.read_block(lazy=lazy, )
-
     def read_block(self, lazy=False, **kargs):
         '''
         Reads a block from the simple spike data file "fname" generated
@@ -153,13 +142,8 @@ class BrainwareF32IO(BaseIO):
         block = self._blk
 
         # create the objects to store other objects
-        chx = ChannelIndex(file_origin=self._filename,
-                           index=np.array([], dtype=np.int))
-        self.__unit = Unit(file_origin=self._filename)
-
-        # load objects into their containers
-        block.channel_indexes.append(chx)
-        chx.units.append(self.__unit)
+        self.__unit_group =  Group(file_origin=self._filename)
+        block.groups.append(self.__unit_group)
 
         # initialize values
         self.__t_stop = None
@@ -286,7 +270,7 @@ class BrainwareF32IO(BaseIO):
                            file_origin=self._filename)
 
         self.__seg.spiketrains = [train]
-        self.__unit.spiketrains.append(train)
+        self.__unit_group.spiketrains.append(train)
         self._blk.segments.append(self.__seg)
 
         # set an empty segment
