@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 from neo.io.basefromrawio import BaseFromRaw
 from neo.rawio.elanrawio import ElanRawIO
 
@@ -14,8 +12,8 @@ class ElanIO(ElanRawIO, BaseFromRaw):
 
     https://elan.lyon.inserm.fr
     """
-    _prefered_signal_group_mode = 'split-all'
-    # _prefered_signal_group_mode = 'group-by-same-units'
+    _prefered_signal_group_mode = 'group-by-same-units'
+    _default_group_mode_have_change_in_0_9 = True
 
     def __init__(self, filename):
         ElanRawIO.__init__(self, filename=filename)
