@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 from neo.io.basefromrawio import BaseFromRaw
 from neo.rawio.plexonrawio import PlexonRawIO
 
@@ -15,7 +13,8 @@ class PlexonIO(PlexonRawIO, BaseFromRaw):
     Compatible with versions 100 to 106.
     Other versions have not been tested.
     """
-    _prefered_signal_group_mode = 'split-all'
+    _prefered_signal_group_mode = 'group-by-same-units'
+    _default_group_mode_have_change_in_0_9 = True
 
     def __init__(self, filename):
         PlexonRawIO.__init__(self, filename=filename)
