@@ -1,18 +1,20 @@
-# -*- coding: utf-8 -*-
 """
 Tools for IO coder:
   * Creating RecordingChannel and making links with AnalogSignals and
     SPikeTrains
 """
 
-import collections
+try:
+    from collections.abc import MutableSequence
+except ImportError:
+    from collections import MutableSequence
 
 import numpy as np
 
 from neo.core import (AnalogSignal, Block,
                       Epoch, Event,
                       IrregularlySampledSignal,
-                      ChannelIndex,
+                      ChannelIndex, Group, ChannelView,
                       Segment, SpikeTrain, Unit)
 
 
@@ -79,16 +81,16 @@ from neo.core import (AnalogSignal, Block,
 #        return D.items()  # Python 3
 
 
-class LazyList(collections.MutableSequence):
+class LazyList(MutableSequence):
     """ An enhanced list that can load its members on demand. Behaves exactly
     like a regular list for members that are Neo objects. Each item should
     contain the information that ``load_lazy_cascade`` needs to load the
     respective object.
     """
-    _container_objects = set(
-        [Block, Segment, ChannelIndex, Unit])
+    _container_objects = {
+        Block, Segment, ChannelIndex, Unit, Group}
     _neo_objects = _container_objects.union(
-        [AnalogSignal, Epoch, Event,
+        [AnalogSignal, Epoch, Event, ChannelView,
          IrregularlySampledSignal, SpikeTrain])
 
     def __init__(self, io, lazy, items=None):
