@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 from neo.io.basefromrawio import BaseFromRaw
 from neo.rawio.winedrrawio import WinEdrRawIO
 
@@ -12,7 +10,8 @@ class WinEdrIO(WinEdrRawIO, BaseFromRaw):
     WinEdr is free:
     http://spider.science.strath.ac.uk/sipbs/software.htm
     """
-    _prefered_signal_group_mode = 'split-all'
+    _prefered_signal_group_mode = 'group-by-same-units'
+    _default_group_mode_have_change_in_0_9 = True
 
     def __init__(self, filename):
         WinEdrRawIO.__init__(self, filename=filename)
