@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Module for reading data from files in the Blackrock in raw format.
 
@@ -7,7 +6,7 @@ This work is based on:
   * Michael Denker, Lyuba Zehl - second version
   * Samuel Garcia - third version
   * Lyuba Zehl, Michael Denker - fourth version
-  * Samuel Garcia - fifth version
+  * Samuel Garcia, Julia Srenger - fifth version
 
 This IO supports reading only.
 This IO is able to read:
@@ -57,7 +56,6 @@ TODO:
     (file spec 2.1 and 2.2)
 """
 
-from __future__ import absolute_import, division, print_function
 
 import datetime
 import os
@@ -104,13 +102,12 @@ class BlackrockRawIO(BaseRawIO):
             File name of the .nev file (without extension). If None,
             filename is used.
             Default: None.
-        nsx_to_load (int):
-            ID of nsX file from which to load data, e.g., if set to
-            5 only data from the ns5 file are loaded. If None, the Io
-            will take the maximum if file is present.
-            Contrary to previsous version of the IO:
-              * nsx_to_load is not a list
-              * must be set at the init before parse_header()
+        nsx_to_load (int, list, 'max', 'all' (=None)) default None:
+            IDs of nsX file from which to load data, e.g., if set to
+            5 only data from the ns5 file are loaded.
+            If 'all', then all nsX will be loaded.
+            Contrary to previsous version of the IO  (<0.7), nsx_to_load
+            must be set at the init before parse_header().
 
     Examples:
         >>> reader = BlackrockRawIO(filename='FileSpec2.3001', nsx_to_load=5)
@@ -297,12 +294,34 @@ class BlackrockRawIO(BaseRawIO):
             # for nsxdef get_analogsignal_shape(self, block_index, seg_index):
             self.__nsx_data_header[nsx_nb] = self.__nsx_dataheader_reader[spec](nsx_nb)
 
-        # We can load only one for one class instance
-        if self.nsx_to_load is None and len(self._avail_nsx) > 0:
-            self.nsx_to_load = max(self._avail_nsx)
+        # nsx_to_load can be either int, list, 'max', all' (aka None)
+        # here make a list only
+        if self.nsx_to_load is None or self.nsx_to_load == 'all':
+            self.nsx_to_load = list(self._avail_nsx)
+        elif self.nsx_to_load == 'max':
+            if len(self._avail_nsx):
+                self.nsx_to_load = [max(self._avail_nsx)]
+            else:
+                self.nsx_to_load = []
+        elif isinstance(self.nsx_to_load, int):
+            self.nsx_to_load = [self.nsx_to_load]
+        elif isinstance(self.nsx_to_load, list):
+            pass
+        else:
+            raise(ValueError('nsx_to_load is wrong'))
 
-        if self.nsx_to_load is not None and \
-                self.__nsx_spec[self.nsx_to_load] == '2.1' and \
+        assert all(nsx_nb in self._avail_nsx for nsx_nb in self.nsx_to_load),\
+                                    'nsx_to_load do not match available nsx list'
+
+        # check that all files come from the same specification
+        all_spec = [self.__nsx_spec[nsx_nb] for nsx in self.nsx_to_load]
+        if self._avail_files['nev']:
+            all_spec.append(self.__nev_spec)
+        assert all(all_spec[0] == spec for spec in all_spec), \
+            "Files don't have the same internal version"
+
+        if len(self.nsx_to_load) > 0 and \
+                self.__nsx_spec[self.nsx_to_load[0]] == '2.1' and \
                 not self._avail_files['nev']:
             pass
             # Because rescaling to volts requires information from nev file (dig_factor)
@@ -312,69 +331,84 @@ class BlackrockRawIO(BaseRawIO):
         # This requires nsX to be parsed already
         # Needs to be called when no nsX are available as well in order to warn the user
         if self._avail_files['nev']:
-            self.__match_nsx_and_nev_segment_ids(self.nsx_to_load)
+            for nsx_nb in self.nsx_to_load:
+                self.__match_nsx_and_nev_segment_ids(nsx_nb)
 
-        if self.nsx_to_load is not None:
-            spec = self.__nsx_spec[self.nsx_to_load]
-            self.nsx_data = self.__nsx_data_reader[spec](self.nsx_to_load)
+        # usefull to get local channel index in nsX from the global channel index
+        local_sig_indexes = []
 
-            self._nb_segment = len(self.nsx_data)
+        self.nsx_datas = {}
+        self.sig_sampling_rates = {}
+        if len(self.nsx_to_load) > 0:
+            for nsx_nb in self.nsx_to_load:
+                spec = self.__nsx_spec[nsx_nb]
+                self.nsx_datas[nsx_nb] = self.__nsx_data_reader[spec](nsx_nb)
 
-            sig_sampling_rate = float(
-                main_sampling_rate / self.__nsx_basic_header[self.nsx_to_load]['period'])
+                sr = float(main_sampling_rate / self.__nsx_basic_header[nsx_nb]['period'])
+                self.sig_sampling_rates[nsx_nb] = sr
 
-            if spec in ['2.2', '2.3']:
-                ext_header = self.__nsx_ext_header[self.nsx_to_load]
-            elif spec == '2.1':
-                ext_header = []
-                keys = ['labels', 'units', 'min_analog_val',
-                        'max_analog_val', 'min_digital_val', 'max_digital_val']
-                params = self.__nsx_params[spec](self.nsx_to_load)
-                for i in range(len(params['labels'])):
-                    d = {}
-                    for key in keys:
-                        d[key] = params[key][i]
-                    ext_header.append(d)
-
-            for i, chan in enumerate(ext_header):
                 if spec in ['2.2', '2.3']:
-                    ch_name = chan['electrode_label'].decode()
-                    ch_id = chan['electrode_id']
-                    units = chan['units'].decode()
+                    ext_header = self.__nsx_ext_header[nsx_nb]
                 elif spec == '2.1':
-                    ch_name = chan['labels']
-                    ch_id = self.__nsx_ext_header[self.nsx_to_load][i]['electrode_id']
-                    units = chan['units']
-                sig_dtype = 'int16'
-                # max_analog_val/min_analog_val/max_digital_val/min_analog_val are int16!!!!!
-                # dangarous situation so cast to float everyone
-                if np.isnan(float(chan['min_analog_val'])):
-                    gain = 1
-                    offset = 0
-                else:
-                    gain = (float(chan['max_analog_val']) - float(chan['min_analog_val'])) / \
-                           (float(chan['max_digital_val']) - float(chan['min_digital_val']))
-                    offset = -float(chan['min_digital_val']) \
-                        * gain + float(chan['min_analog_val'])
-                group_id = 0
-                sig_channels.append((ch_name, ch_id, sig_sampling_rate, sig_dtype,
-                                     units, gain, offset, group_id,))
+                    ext_header = []
+                    keys = ['labels', 'units', 'min_analog_val',
+                            'max_analog_val', 'min_digital_val', 'max_digital_val']
+                    params = self.__nsx_params[spec](nsx_nb)
+                    for i in range(len(params['labels'])):
+                        d = {}
+                        for key in keys:
+                            d[key] = params[key][i]
+                        ext_header.append(d)
+
+                for i, chan in enumerate(ext_header):
+                    if spec in ['2.2', '2.3']:
+                        ch_name = chan['electrode_label'].decode()
+                        ch_id = chan['electrode_id']
+                        units = chan['units'].decode()
+                    elif spec == '2.1':
+                        ch_name = chan['labels']
+                        ch_id = self.__nsx_ext_header[nsx_nb][i]['electrode_id']
+                        units = chan['units']
+                    sig_dtype = 'int16'
+                    # max_analog_val/min_analog_val/max_digital_val/min_analog_val are int16!!!!!
+                    # dangarous situation so cast to float everyone
+                    if np.isnan(float(chan['min_analog_val'])):
+                        gain = 1
+                        offset = 0
+                    else:
+                        gain = (float(chan['max_analog_val']) - float(chan['min_analog_val'])) / \
+                               (float(chan['max_digital_val']) - float(chan['min_digital_val']))
+                        offset = -float(chan['min_digital_val']) \
+                            * gain + float(chan['min_analog_val'])
+                    group_id = nsx_nb
+                    sig_channels.append((ch_name, ch_id, sr, sig_dtype,
+                                         units, gain, offset, group_id,))
+                local_sig_indexes.extend(range(len(ext_header)))
+
+            self._local_sig_indexes = np.array(local_sig_indexes)
+
+            # check nb segment per nsx
+            nb_segments_for_nsx = [len(self.nsx_datas[nsx_nb]) for nsx_nb in self.nsx_to_load]
+            assert all(nb == nb_segments_for_nsx[0] for nb in nb_segments_for_nsx),\
+                                                    'Segment nb not consistanent across nsX files'
+            self._nb_segment = nb_segments_for_nsx[0]
 
             self.__delete_empty_segments()
 
             # t_start/t_stop for segment are given by nsx limits or nev limits
-            self._sigs_t_starts = []
+            self._sigs_t_starts = {nsx_nb: [] for nsx_nb in self.nsx_to_load}
             self._seg_t_starts, self._seg_t_stops = [], []
             for data_bl in range(self._nb_segment):
-                length = self.nsx_data[data_bl].shape[0]
-                if self.__nsx_data_header[self.nsx_to_load] is None:
-                    t_start = 0.
-                else:
-                    t_start = self.__nsx_data_header[self.nsx_to_load][data_bl]['timestamp'] / \
-                        self.__nsx_basic_header[self.nsx_to_load]['timestamp_resolution']
-                self._sigs_t_starts.append(t_start)
-
-                t_stop = t_start + length / sig_sampling_rate
+                t_stop = 0.
+                for nsx_nb in self.nsx_to_load:
+                    length = self.nsx_datas[nsx_nb][data_bl].shape[0]
+                    if self.__nsx_data_header[nsx_nb] is None:
+                        t_start = 0.
+                    else:
+                        t_start = self.__nsx_data_header[nsx_nb][data_bl]['timestamp'] / \
+                            self.__nsx_basic_header[nsx_nb]['timestamp_resolution']
+                    t_stop = max(t_stop, t_start + length / self.sig_sampling_rates[nsx_nb])
+                    self._sigs_t_starts[nsx_nb].append(t_start)
 
                 if self._avail_files['nev']:
                     max_nev_time = 0
@@ -473,9 +507,11 @@ class BlackrockRawIO(BaseRawIO):
                     chidx_ann['connector_pinID'] = neuevwav['connector_pin'][get_idx]
                     chidx_ann['nev_dig_factor'] = neuevwav['digitization_factor'][get_idx]
                     chidx_ann['nev_energy_threshold'] = neuevwav['energy_threshold'][
-                        get_idx] * pq.uV
-                    chidx_ann['nev_hi_threshold'] = neuevwav['hi_threshold'][get_idx] * pq.uV
-                    chidx_ann['nev_lo_threshold'] = neuevwav['lo_threshold'][get_idx] * pq.uV
+                        get_idx] * chidx_ann['nev_dig_factor'] / 1000 * pq.uV
+                    chidx_ann['nev_hi_threshold'] = neuevwav['hi_threshold'][get_idx] * chidx_ann[
+                        'nev_dig_factor'] / 1000 * pq.uV
+                    chidx_ann['nev_lo_threshold'] = neuevwav['lo_threshold'][get_idx] * chidx_ann[
+                        'nev_dig_factor'] / 1000 * pq.uV
                     chidx_ann['nb_sorted_units'] = neuevwav['nb_sorted_units'][get_idx]
                     chidx_ann['waveform_size'] = self.__waveform_size[self.__nev_spec](
                     )[sig_channels[c]['id']] * self.__nev_params('waveform_time_unit')
@@ -495,7 +531,7 @@ class BlackrockRawIO(BaseRawIO):
                         chidx_ann['nev_lo_freq_order'] = neuevflt['lo_freq_order'][get_idx]
                         chidx_ann['nev_lo_freq_type'] = flt_type[neuevflt['lo_freq_type'][
                             get_idx]]
-            if self.__nsx_spec[self.nsx_to_load] in ['2.2', '2.3'] and self.__nsx_ext_header:
+            if self.__nsx_spec[self.nsx_to_load[0]] in ['2.2', '2.3'] and self.__nsx_ext_header:
                 # It does not matter which nsX file to ask for this info
                 k = list(self.__nsx_ext_header.keys())[0]
                 if sig_channels[c]['id'] in self.__nsx_ext_header[k]['electrode_id']:
@@ -530,12 +566,13 @@ class BlackrockRawIO(BaseRawIO):
                 seg_ann['rec_datetime'] = rec_datetime
 
             for c in range(sig_channels.size):
+                nsx_nb = sig_channels['group_id'][c]
                 anasig_an = seg_ann['signals'][c]
                 desc = "AnalogSignal {} from channel_id: {}, label: {}, nsx: {}".format(
-                    c, sig_channels['id'][c], sig_channels['name'][c], self.nsx_to_load)
+                    c, sig_channels['id'][c], sig_channels['name'][c], nsx_nb)
                 anasig_an['description'] = desc
-                anasig_an['file_origin'] = self._filenames['nsx'] + '.ns' + str(self.nsx_to_load)
-                anasig_an['nsx'] = self.nsx_to_load
+                anasig_an['file_origin'] = self._filenames['nsx'] + '.ns' + str(nsx_nb)
+                anasig_an['nsx'] = nsx_nb
                 chidx_ann = self.raw_annotations['signal_channels'][c]
                 chidx_ann['description'] = 'Container for Units and AnalogSignals of ' \
                                            'one recording channel across segments.'
@@ -573,20 +610,33 @@ class BlackrockRawIO(BaseRawIO):
     def _segment_t_stop(self, block_index, seg_index):
         return self._seg_t_stops[seg_index]
 
+    def _get_nsx_and_local_indexes(self, channel_indexes):
+        # internal helper to get nsx number and local channel index
+        # from global channel indexes
+        # when this is called channell_indexes are alwas in the same group_id
+        # this is checked at BaseRaw level
+        if channel_indexes is None:
+            channel_indexes = slice(None)
+        nsx_nb = self.header['signal_channels'][channel_indexes]['group_id'][0]
+        if channel_indexes is None:
+            local_indexes = slice(None)
+        else:
+            local_indexes = self._local_sig_indexes[channel_indexes]
+        return nsx_nb, local_indexes
+
     def _get_signal_size(self, block_index, seg_index, channel_indexes):
-        memmap_data = self.nsx_data[seg_index]
+        nsx_nb, local_indexes = self._get_nsx_and_local_indexes(channel_indexes)
+        memmap_data = self.nsx_datas[nsx_nb][seg_index]
         return memmap_data.shape[0]
 
     def _get_signal_t_start(self, block_index, seg_index, channel_indexes):
-        return self._sigs_t_starts[seg_index]
+        nsx_nb, local_indexes = self._get_nsx_and_local_indexes(channel_indexes)
+        return self._sigs_t_starts[nsx_nb][seg_index]
 
     def _get_analogsignal_chunk(self, block_index, seg_index, i_start, i_stop, channel_indexes):
-        assert block_index == 0
-        memmap_data = self.nsx_data[seg_index]
-
-        if channel_indexes is None:
-            channel_indexes = slice(None)
-        sig_chunk = memmap_data[i_start:i_stop, channel_indexes]
+        nsx_nb, local_indexes = self._get_nsx_and_local_indexes(channel_indexes)
+        memmap_data = self.nsx_datas[nsx_nb][seg_index]
+        sig_chunk = memmap_data[i_start:i_stop, local_indexes]
         return sig_chunk
 
     def _spike_count(self, block_index, seg_index, unit_index):
@@ -698,15 +748,11 @@ class BlackrockRawIO(BaseRawIO):
             encoding = {0: 'latin_1', 1: 'utf_16', 255: 'latin_1'}
             labels = [data[ev_dict['field']].decode(
                 encoding[data['char_set']]) for data in events_data]
-            # Only ASCII can be allowed due to using numpy
-            # labels.astype('S') forces to use bytes in BaseFromRaw 401, in read_segment
-            # This is not recommended
-            # TODO: Maybe switch to astype('U')
-            labels = np.array([data.encode('ASCII', errors='ignore') for data in labels])
+            labels = np.array(labels, dtype='U')
         else:
             events_data, event_segment_ids = self.nev_data['NonNeural']
             ev_dict = self.__nonneural_evdicts[self.__nev_spec](events_data)[name]
-            labels = events_data[ev_dict['field']]
+            labels = events_data[ev_dict['field']].astype('U')
 
         mask = ev_dict['mask'] & (event_segment_ids == seg_index)
         timestamp = events_data[mask]['timestamp']
@@ -748,7 +794,7 @@ class BlackrockRawIO(BaseRawIO):
         if nsx_file_id['file_id'].decode() == 'NEURALSG':
             spec = '2.1'
         elif nsx_file_id['file_id'].decode() == 'NEURALCD':
-            spec = '{0}.{1}'.format(
+            spec = '{}.{}'.format(
                 nsx_file_id['ver_major'], nsx_file_id['ver_minor'])
         else:
             raise IOError('Unsupported NSX file type.')
@@ -769,10 +815,10 @@ class BlackrockRawIO(BaseRawIO):
 
         nev_file_id = np.fromfile(filename, count=1, dtype=dt0)[0]
         if nev_file_id['file_id'].decode() == 'NEURALEV':
-            spec = '{0}.{1}'.format(
+            spec = '{}.{}'.format(
                 nev_file_id['ver_major'], nev_file_id['ver_minor'])
         else:
-            raise IOError('NEV file type {0} is not supported'.format(
+            raise IOError('NEV file type {} is not supported'.format(
                 nev_file_id['file_id']))
 
         return spec
@@ -1097,7 +1143,7 @@ class BlackrockRawIO(BaseRawIO):
         dt0 = [
             ('timestamp', 'uint32'),
             ('packet_id', 'uint16'),
-            ('value', 'S{0}'.format(data_size - 6))]
+            ('value', 'S{}'.format(data_size - 6))]
 
         raw_data = np.memmap(filename, offset=header_size, dtype=dt0, mode='r')
 
@@ -1172,7 +1218,7 @@ class BlackrockRawIO(BaseRawIO):
         if self.__nev_spec == '2.3':
             nsx_offset = self.__nsx_data_header[nsx_nb][0]['timestamp']
             # Multiples of 1/30.000s that pass between two nsX samples
-            nsx_period = self.__nsx_basic_header[self.nsx_to_load]['period']
+            nsx_period = self.__nsx_basic_header[nsx_nb]['period']
             # NSX segments needed as dict and list
             nonempty_nsx_segments = {}
             list_nonempty_nsx_segments = []
@@ -1200,14 +1246,14 @@ class BlackrockRawIO(BaseRawIO):
                     end_of_current_nsx_seg = seg['timestamp'] + \
                         seg['nb_data_points'] * self.__nsx_basic_header[nsx_nb]['period']
 
-                    mask_after_seg = [(ev_ids == i)
-                                      & (data['timestamp'] > end_of_current_nsx_seg + nsx_period)]
+                    mask_after_seg = (ev_ids == i) & \
+                                    (data['timestamp'] > end_of_current_nsx_seg + nsx_period)
 
                     # Show warning if spikes do not fit any segment (+- 1 sampling 'tick')
                     # Spike should belong to segment before
-                    mask_outside = [(ev_ids == i)
-                                    & (data['timestamp'] < int(seg['timestamp'])
-                                       - nsx_offset - nsx_period)]
+                    mask_outside = (ev_ids == i) & \
+                            (data['timestamp'] < int(seg['timestamp']) - nsx_offset - nsx_period)
+
                     if len(data[mask_outside]) > 0:
                         warnings.warn("Spikes outside any segment. Detected on segment #{}".
                                       format(i))
@@ -1223,7 +1269,7 @@ class BlackrockRawIO(BaseRawIO):
                         # Warning if spikes are after last segment
                         if i == len(list_nonempty_nsx_segments) - 1:
                             timestamp_resolution = self.__nsx_params[self.__nsx_spec[
-                                self.nsx_to_load]]('timestamp_resolution', self.nsx_to_load)
+                                nsx_nb]]('timestamp_resolution', nsx_nb)
                             time_after_seg = (data[mask_after_seg]['timestamp'][-1]
                                               - end_of_current_nsx_seg) / timestamp_resolution
                             warnings.warn("Spikes {}s after last segment.".format(time_after_seg))
@@ -1248,7 +1294,7 @@ class BlackrockRawIO(BaseRawIO):
             # consistency check: same number of segments for nsx and nev data
             assert nb_possible_nev_segments == len(nonempty_nsx_segments), \
                 ('Inconsistent ns{0} and nev file. {1} segments present in .nev file, but {2} in '
-                 'ns{0} file.'.format(self.nsx_to_load, nb_possible_nev_segments,
+                 'ns{0} file.'.format(nsx_nb, nb_possible_nev_segments,
                                       len(nonempty_nsx_segments)))
 
             new_nev_segment_id_mapping = dict(zip(range(nb_possible_nev_segments),
@@ -1459,7 +1505,7 @@ class BlackrockRawIO(BaseRawIO):
                     ('analog_input_channel_3', 'int16'),
                     ('analog_input_channel_4', 'int16'),
                     ('analog_input_channel_5', 'int16'),
-                    ('unused', 'S{0}'.format(data_size - 20))],
+                    ('unused', 'S{}'.format(data_size - 20))],
                 # Version>=2.3
                 'b': [
                     ('timestamp', 'uint32'),
@@ -1467,14 +1513,14 @@ class BlackrockRawIO(BaseRawIO):
                     ('packet_insertion_reason', 'uint8'),
                     ('reserved', 'uint8'),
                     ('digital_input', 'uint16'),
-                    ('unused', 'S{0}'.format(data_size - 10))]},
+                    ('unused', 'S{}'.format(data_size - 10))]},
             'Spikes': {
                 'a': [
                     ('timestamp', 'uint32'),
                     ('packet_id', 'uint16'),
                     ('unit_class_nb', 'uint8'),
                     ('reserved', 'uint8'),
-                    ('waveform', 'S{0}'.format(data_size - 8))]},
+                    ('waveform', 'S{}'.format(data_size - 8))]},
             'Comments': {
                 'a': [
                     ('timestamp', 'uint32'),
@@ -1482,7 +1528,7 @@ class BlackrockRawIO(BaseRawIO):
                     ('char_set', 'uint8'),
                     ('flag', 'uint8'),
                     ('color', 'uint32'),
-                    ('comment', 'S{0}'.format(data_size - 12))]},
+                    ('comment', 'S{}'.format(data_size - 12))]},
             'VideoSync': {
                 'a': [
                     ('timestamp', 'uint32'),
@@ -1512,7 +1558,7 @@ class BlackrockRawIO(BaseRawIO):
                     ('timestamp', 'uint32'),
                     ('packet_id', 'uint16'),
                     ('config_change_type', 'uint16'),
-                    ('config_changed', 'S{0}'.format(data_size - 8))]}}
+                    ('config_changed', 'S{}'.format(data_size - 8))]}}
 
         return __nev_data_types
 
@@ -1534,7 +1580,7 @@ class BlackrockRawIO(BaseRawIO):
             'max_res': self.__nev_basic_header['timestamp_resolution'],
             'channel_ids': self.__nev_ext_header[b'NEUEVWAV']['electrode_id'],
             'channel_labels': self.__channel_labels[self.__nev_spec](),
-            'event_unit': pq.CompoundUnit("1.0/{0} * s".format(
+            'event_unit': pq.CompoundUnit("1.0/{} * s".format(
                 self.__nev_basic_header['timestamp_resolution'])),
             'nb_units': dict(zip(
                 self.__nev_ext_header[b'NEUEVWAV']['electrode_id'],
@@ -1547,7 +1593,7 @@ class BlackrockRawIO(BaseRawIO):
             'waveform_dtypes': self.__get_waveforms_dtype(),
             'waveform_sampling_rate':
                 self.__nev_basic_header['sample_resolution'] * pq.Hz,
-            'waveform_time_unit': pq.CompoundUnit("1.0/{0} * s".format(
+            'waveform_time_unit': pq.CompoundUnit("1.0/{} * s".format(
                 self.__nev_basic_header['sample_resolution'])),
             'waveform_unit': pq.uV}
 
@@ -1621,7 +1667,7 @@ class BlackrockRawIO(BaseRawIO):
         # get the dtype of waveform (this is stupidly complicated)
         if self.__is_set(
                 np.array(self.__nev_basic_header['additionnal_flags']), 0):
-            dtype_waveforms = dict((k, 'int16') for k in all_el_ids)
+            dtype_waveforms = {k: 'int16' for k in all_el_ids}
         else:
             # extract bytes per waveform
             waveform_bytes = \
@@ -1662,9 +1708,9 @@ class BlackrockRawIO(BaseRawIO):
         wf_dtypes = self.__get_waveforms_dtype()
         nb_bytes_wf = self.__nev_basic_header['bytes_in_data_packets'] - 8
 
-        wf_sizes = dict([
-            (ch, int(nb_bytes_wf / np.dtype(dt).itemsize)) for ch, dt in
-            wf_dtypes.items()])
+        wf_sizes = {
+            ch: int(nb_bytes_wf / np.dtype(dt).itemsize) for ch, dt in
+            wf_dtypes.items()}
 
         return wf_sizes
 
@@ -1689,7 +1735,7 @@ class BlackrockRawIO(BaseRawIO):
 
         # TODO: Double check if this is the correct assumption (10 samples)
         # default value: threshold crossing after 10 samples of waveform
-        wf_left_sweep = dict([(ch, 10 * wf_t_unit) for ch in all_ch])
+        wf_left_sweep = {ch: 10 * wf_t_unit for ch in all_ch}
 
         # non-default: threshold crossing at center of waveform
         # wf_size = self.__nev_params('waveform_size')
@@ -1756,7 +1802,7 @@ class BlackrockRawIO(BaseRawIO):
             'timestamp_resolution': 30000,
             'bytes_in_headers': bytes_in_headers,
             'sampling_rate': 30000 / self.__nsx_basic_header[nsx_nb]['period'] * pq.Hz,
-            'time_unit': pq.CompoundUnit("1.0/{0}*s".format(
+            'time_unit': pq.CompoundUnit("1.0/{}*s".format(
                 30000 / self.__nsx_basic_header[nsx_nb]['period']))}
 
         # Returns complete dictionary because then it does not need to be called so often
@@ -1787,7 +1833,7 @@ class BlackrockRawIO(BaseRawIO):
             'sampling_rate':
                 self.__nsx_basic_header[nsx_nb]['timestamp_resolution']
                 / self.__nsx_basic_header[nsx_nb]['period'] * pq.Hz,
-            'time_unit': pq.CompoundUnit("1.0/{0}*s".format(
+            'time_unit': pq.CompoundUnit("1.0/{}*s".format(
                 self.__nsx_basic_header[nsx_nb]['timestamp_resolution']
                 / self.__nsx_basic_header[nsx_nb]['period']))}
 
@@ -1805,26 +1851,23 @@ class BlackrockRawIO(BaseRawIO):
             'digital_input_port': {
                 'name': 'digital_input_port',
                 'field': 'digital_input',
-                'mask': self.__is_set(data['packet_insertion_reason'], 0)
-                        & ~self.__is_set(data['packet_insertion_reason'], 7),
+                'mask': data['packet_insertion_reason'] == 1,
                 'desc': "Events of the digital input port"},
             'serial_input_port': {
                 'name': 'serial_input_port',
                 'field': 'digital_input',
-                'mask':
-                    self.__is_set(data['packet_insertion_reason'], 0)
-                    & self.__is_set(data['packet_insertion_reason'], 7),
+                'mask': data['packet_insertion_reason'] == 129,
                 'desc': "Events of the serial input port"}}
 
         # analog input events via threshold crossings
         for ch in range(5):
             event_types.update({
-                'analog_input_channel_{0}'.format(ch + 1): {
-                    'name': 'analog_input_channel_{0}'.format(ch + 1),
-                    'field': 'analog_input_channel_{0}'.format(ch + 1),
+                'analog_input_channel_{}'.format(ch + 1): {
+                    'name': 'analog_input_channel_{}'.format(ch + 1),
+                    'field': 'analog_input_channel_{}'.format(ch + 1),
                     'mask': self.__is_set(
                         data['packet_insertion_reason'], ch + 1),
-                    'desc': "Values of analog input channel {0} in mV "
+                    'desc': "Values of analog input channel {} in mV "
                             "(+/- 5000)".format(ch + 1)}})
 
         # TODO: define field and desc
@@ -1846,41 +1889,38 @@ class BlackrockRawIO(BaseRawIO):
         segment in the nsX data.
         """
 
-        nb_empty_segments = 0
+        # Discard empty segments
+        removed_seg = []
         for data_bl in range(self._nb_segment):
-            length = self.nsx_data[data_bl].shape[0]
-            # Discard empty segments
-            if length < 2:
-                nb_empty_segments += 1
-                self.nsx_data.pop(data_bl)
-                self.__nsx_data_header[self.nsx_to_load].pop(data_bl)
-                continue
+            keep_seg = True
+            for nsx_nb in self.nsx_to_load:
+                length = self.nsx_datas[nsx_nb][data_bl].shape[0]
+                keep_seg = keep_seg and (length >= 2)
+
+            if not keep_seg:
+                removed_seg.append(data_bl)
+                for nsx_nb in self.nsx_to_load:
+                    self.nsx_datas[nsx_nb].pop(data_bl)
+                    self.__nsx_data_header[nsx_nb].pop(data_bl)
 
         # Keys need to be increasing from 0 to maximum in steps of 1
-        # To ensure this after removing empty segments, some keys need to be changed
-        for i in range(self._nb_segment):
-            # If this key does not exist, reduce all following keys by 1,
-            # so the keys become a range starting from 0
-            if i not in self.nsx_data:
-                self.nsx_data = {key - 1 if key > i else key: value for (key, value) in
-                                 self.nsx_data.items()}
-                data_header = self.__nsx_data_header[self.nsx_to_load]
-                data_header = {key - 1 if key > i else key: value for (key, value) in
-                               data_header.items()}
-                self.__nsx_data_header[self.nsx_to_load] = data_header
+        # To ensure this after removing empty segments, some keys need to be re mapped
+        for i in removed_seg[::-1]:
+            for j in range(i + 1, self._nb_segment):
+                # remap nsx seg index
+                for nsx_nb in self.nsx_to_load:
+                    data = self.nsx_datas[nsx_nb].pop(j)
+                    self.nsx_datas[nsx_nb][j - 1] = data
+
+                    data_header = self.__nsx_data_header[nsx_nb].pop(j)
+                    self.__nsx_data_header[nsx_nb][j - 1] = data_header
+
                 # Also remap nev data, ev_ids are the equivalent to keys above
                 if self._avail_files['nev']:
                     for k, (data, ev_ids) in self.nev_data.items():
-                        # If this segment id does not exist, reduce all following by 1
-                        # to make them a range starting from 0
-                        if i not in ev_ids:
-                            ev_ids[ev_ids > i] -= 1
-                        else:
-                            raise ValueError("A segment in nsX was omitted but there were"
-                                             "spikes matched to it.")
+                        ev_ids[ev_ids == j] -= 1
 
-        # Empty segments were discarded, thus there may be less segments now
-        self._nb_segment -= nb_empty_segments
+            self._nb_segment -= 1
 
     def __get_nonneural_evdicts_variant_b(self, data):
         """
