@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 ExampleRawIO is a class of a  fake example.
 This is to be used when coding a new RawIO.
@@ -27,7 +26,7 @@ Rules for creating a new class:
 
   3. Step 3 : Create the neo.io class with the wrapper
     * Create a file in neo/io/ that endith with "io.py"
-    * Create a that hinerits bot yrou RawIO class and BaseFromRaw class
+    * Create a that inherits both your RawIO class and BaseFromRaw class
     * copy/paste from neo/io/exampleio.py
 
   4.Step 4 : IO test
@@ -37,7 +36,6 @@ Rules for creating a new class:
 
 
 """
-from __future__ import unicode_literals, print_function, division, absolute_import
 
 from .baserawio import (BaseRawIO, _signal_channel_dtype, _unit_channel_dtype,
                         _event_channel_dtype)
@@ -56,8 +54,8 @@ class ExampleRawIO(BaseRawIO):
     to develop a new IO module.
 
     Two rules for developers:
-      * Respect the Neo RawIO API (:ref:`_neo_rawio_API`)
-      * Follow :ref:`_io_guiline`
+      * Respect the :ref:`neo_rawio_API`
+      * Follow the :ref:`io_guiline`
 
     This fake IO:
         * have 2 blocks
@@ -309,9 +307,13 @@ class ExampleRawIO(BaseRawIO):
         # it is not always the case
         # we 20 spikes with a sweep of 50 (5ms)
 
+        # trick to get how many spike in the slice
+        ts = self._get_spike_timestamps(block_index, seg_index, unit_index, t_start, t_stop)
+        nb_spike = ts.size
+
         np.random.seed(2205)  # a magic number (my birthday)
-        waveforms = np.random.randint(low=-2 ** 4, high=2 ** 4, size=20 * 50, dtype='int16')
-        waveforms = waveforms.reshape(20, 1, 50)
+        waveforms = np.random.randint(low=-2**4, high=2**4, size=nb_spike * 50, dtype='int16')
+        waveforms = waveforms.reshape(nb_spike, 1, 50)
         return waveforms
 
     def _event_count(self, block_index, seg_index, event_channel_index):
