@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 
 Support for intan tech rhd  and rhs files.
@@ -17,7 +16,6 @@ See:
 Author: Samuel Garcia
 
 """
-from __future__ import print_function, division, absolute_import
 # from __future__ import unicode_literals is not compatible with numpy.dtype both py2 py3
 
 from .baserawio import (BaseRawIO, _signal_channel_dtype, _unit_channel_dtype,
