@@ -1,11 +1,7 @@
-# -*- coding: utf-8 -*-
 '''
 Generate datasets for testing
 '''
 
-# needed for python 3 compatibility
-from __future__ import absolute_import
-
 from datetime import datetime
 
 import numpy as np
@@ -13,7 +9,8 @@ from numpy.random import rand
 import quantities as pq
 
 from neo.core import (AnalogSignal, Block, Epoch, Event, IrregularlySampledSignal, ChannelIndex,
-                      Segment, SpikeTrain, Unit, class_by_name)
+                      Segment, SpikeTrain, Unit, ImageSequence, CircularRegionOfInterest,
+                      RectangularRegionOfInterest, PolygonRegionOfInterest, class_by_name)
 
 from neo.core.baseneo import _container_name
 from neo.core.dataobject import DataObject
@@ -52,7 +49,7 @@ def generate_one_simple_segment(seg_name='segment 0', supported_objects=[], nb_a
 
                                 epoch_types={'animal state': ['Sleep', 'Freeze', 'Escape'],
                                              'light': ['dark', 'lighted']},
-                                epoch_duration_range=[.5, 3.],  
+                                epoch_duration_range=[.5, 3.],
                                 # this should be multiplied by pq.s, no?
 
                                 array_annotations={'valid': np.array([True, False]),
@@ -64,7 +61,8 @@ def generate_one_simple_segment(seg_name='segment 0', supported_objects=[], nb_a
     seg = Segment(name=seg_name)
     if AnalogSignal in supported_objects:
         for a in range(nb_analogsignal):
-            anasig = AnalogSignal(rand(int(sampling_rate * duration)), sampling_rate=sampling_rate,
+            anasig = AnalogSignal(rand(int((sampling_rate * duration).simplified)),
+                                  sampling_rate=sampling_rate,
                                   t_start=t_start, units=pq.mV, channel_index=a,
                                   name='sig %d for segment %s' % (a, seg.name))
             seg.analogsignals.append(anasig)
@@ -92,7 +90,7 @@ def generate_one_simple_segment(seg_name='segment 0', supported_objects=[], nb_a
             evt_size = rand() * np.diff(event_size_range)
             evt_size += event_size_range[0]
             evt_size = int(evt_size)
-            labels = np.array(labels, dtype='S')
+            labels = np.array(labels, dtype='U')
             labels = labels[(rand(evt_size) * len(labels)).astype('i')]
             evt = Event(times=rand(evt_size) * duration, labels=labels)
             # Randomly generate array_annotations from given options
@@ -112,7 +110,7 @@ def generate_one_simple_segment(seg_name='segment 0', supported_objects=[], nb_a
                 dur += epoch_duration_range[0]
                 durations.append(dur)
                 t = t + dur
-            labels = np.array(labels, dtype='S')
+            labels = np.array(labels, dtype='U')
             labels = labels[(rand(len(times)) * len(labels)).astype('i')]
             assert len(times) == len(durations)
             assert len(times) == len(labels)
@@ -166,14 +164,14 @@ def get_fake_value(name, datatype, dim=0, dtype='float', seed=None, units=None,
         obj = obj.__name__
 
     if (name in ['name', 'file_origin', 'description'] and (datatype != str or dim)):
-        raise ValueError('%s must be str, not a %sD %s' % (name, dim, datatype))
+        raise ValueError('{} must be str, not a {}D {}'.format(name, dim, datatype))
 
     if name == 'file_origin':
         return 'test_file.txt'
     if name == 'name':
-        return '%s%s' % (obj, get_fake_value('', datatype, seed=seed))
+        return '{}{}'.format(obj, get_fake_value('', datatype, seed=seed))
     if name == 'description':
-        return 'test %s %s' % (obj, get_fake_value('', datatype, seed=seed))
+        return 'test {} {}'.format(obj, get_fake_value('', datatype, seed=seed))
 
     if seed is not None:
         np.random.seed(seed)
@@ -188,7 +186,7 @@ def get_fake_value(name, datatype, dim=0, dtype='float', seed=None, units=None,
         return datetime.fromtimestamp(1000000000 * np.random.random())
 
     if (name in ['t_start', 't_stop', 'sampling_rate'] and (datatype != pq.Quantity or dim)):
-        raise ValueError('%s must be a 0D Quantity, not a %sD %s' % (name, dim, datatype))
+        raise ValueError('{} must be a 0D Quantity, not a {}D {}'.format(name, dim, datatype))
 
     # only put array types below here
 
@@ -208,10 +206,14 @@ def get_fake_value(name, datatype, dim=0, dtype='float', seed=None, units=None,
         data = np.array(0.0)
     elif name == 't_stop':
         data = np.array(1.0)
-    elif n and name == 'channel_indexes':
+    elif n and name in ['channel_indexes', 'channel_ids']:
         data = np.arange(n)
+    elif n and name == 'coordinates':
+        data = np.arange(0, 2*n).reshape((n, 2))
     elif n and name == 'channel_names':
         data = np.array(["ch%d" % i for i in range(n)])
+    elif n and name == 'index':  # ChannelIndex.index
+        data = np.random.randint(0, n, n)
     elif n and obj == 'AnalogSignal':
         if name == 'signal':
             size = []
@@ -262,7 +264,7 @@ def get_fake_value(name, datatype, dim=0, dtype='float', seed=None, units=None,
         return arr_ann
 
     # we have gone through everything we know, so it must be something invalid
-    raise ValueError('Unknown name/datatype combination %s %s' % (name, datatype))
+    raise ValueError('Unknown name/datatype combination {} {}'.format(name, datatype))
 
 
 def get_fake_values(cls, annotate=True, seed=None, n=None):
@@ -276,8 +278,7 @@ def get_fake_values(cls, annotate=True, seed=None, n=None):
     If annotate is True (default), also add annotations to the values.
     """
 
-    if hasattr(cls,
-               'lower'):  # is this a test that cls is a string? better to use isinstance(cls,
+    if hasattr(cls, 'lower'):  # is this a test that cls is a string? better to use isinstance(cls,
         # basestring), no?
         cls = class_by_name[cls]
     # iseed is needed below for generation of array annotations
@@ -321,8 +322,8 @@ def get_fake_values(cls, annotate=True, seed=None, n=None):
                 n = 1
         elif cls in [SpikeTrain, Event, Epoch]:
             n = len(kwargs['times'])
-        # Array annotate any DataObject
-        if issubclass(cls, DataObject):
+        # Array annotate any DataObject except ImageSequence
+        if issubclass(cls, DataObject) and cls is not ImageSequence:
             new_seed = iseed + 1 if iseed is not None else iseed
             kwargs['array_annotations'] = get_fake_value('array_annotations', dict, seed=new_seed,
                                                          obj=cls, n=n)
@@ -336,7 +337,7 @@ def get_annotations():
     Returns a dict containing the default values for annotations for
     a class from neo.core.
     '''
-    return dict([(str(i), ann) for i, ann in enumerate(TEST_ANNOTATIONS)])
+    return {str(i): ann for i, ann in enumerate(TEST_ANNOTATIONS)}
 
 
 def fake_epoch(seed=None, n=1):
@@ -397,6 +398,8 @@ def fake_neo(obj_type="Block", cascade=True, seed=None, n=1):
         cascade = 'block'
     for i, childname in enumerate(getattr(obj, '_child_objects', [])):
         # we create a few of each class
+        if childname == 'Group':
+            continue  # avoid infinite recursion, since Groups can contain  Groups
         for j in range(n):
             if seed is not None:
                 iseed = 10 * seed + 100 * i + 1000 * j
@@ -408,7 +411,7 @@ def fake_neo(obj_type="Block", cascade=True, seed=None, n=1):
             # if we are creating a block and this is the object's primary
             # parent, don't create the object, we will import it from secondary
             # containers later
-            if (cascade == 'block' and len(child._parent_objects) > 0 
+            if (cascade == 'block' and len(child._parent_objects) > 0
                     and obj_type != child._parent_objects[-1]):
                 continue
             getattr(obj, _container_name(childname)).append(child)
