@@ -1,13 +1,7 @@
-# -*- coding: utf-8 -*-
 """
 Tests of neo.io.axonio
 """
 
-# needed for python 3 compatibility
-from __future__ import absolute_import
-
-import sys
-
 import unittest
 
 from neo.io import AxonIO
@@ -15,15 +9,16 @@ from neo.test.iotest.common_io_test import BaseTestIO
 
 
 class TestAxonIO(BaseTestIO, unittest.TestCase):
-    files_to_test = ['File_axon_1.abf',
-                     'File_axon_2.abf',
-                     'File_axon_3.abf',
-                     'File_axon_4.abf',
-                     'File_axon_5.abf',
-                     'File_axon_6.abf',
-                     'File_axon_7.abf',
-
-                     ]
+    files_to_test = [
+        'File_axon_1.abf',
+        'File_axon_2.abf',
+        'File_axon_3.abf',
+        'File_axon_4.abf',
+        'File_axon_5.abf',
+        'File_axon_6.abf',
+        'File_axon_7.abf',
+        'test_file_edr3.abf',
+    ]
     files_to_download = files_to_test
     ioclass = AxonIO
 
