@@ -1,11 +1,7 @@
-# -*- coding: utf-8 -*-
 """
 Tests of neo.io.blackrockio
 """
 
-# needed for python 3 compatibility
-from __future__ import absolute_import
-
 import unittest
 import warnings
 
@@ -42,7 +38,8 @@ else:
 class CommonTests(BaseTestIO, unittest.TestCase):
     ioclass = BlackrockIO
 
-    files_to_test = ['FileSpec2.3001']
+    files_to_test = ['FileSpec2.3001',
+        'blackrock_2_1/l101210-001']
 
     files_to_download = [
         'FileSpec2.3001.nev',
@@ -93,8 +90,11 @@ class CommonTests(BaseTestIO, unittest.TestCase):
         too_large_tstop = block.segments[0].analogsignals[0].t_stop + 1 * pq.s
         buggy_slice = (-100 * pq.ms, too_large_tstop)
 
-        # this is valid in read_segment because seg_index is specified
-        seg = reader.read_segment(seg_index=0, time_slice=buggy_slice)
+        # this is valid in read_segment
+        seg = reader.read_segment(seg_index=0, time_slice=buggy_slice, strict_slicing=False)
+        # this raise an error
+        with self.assertRaises(AssertionError):
+            seg = reader.read_segment(seg_index=0, time_slice=buggy_slice, strict_slicing=True)
 
         lenb = len(seg.analogsignals[0])
         numspb = len(seg.spiketrains[0])
@@ -109,12 +109,12 @@ class CommonTests(BaseTestIO, unittest.TestCase):
 
         # test 4 Units
         block = reader.read_block(load_waveforms=True,
-                                  units_group_mode='all-in-one')
+                                  signal_group_mode='split-all')
 
         self.assertEqual(len(block.segments[0].analogsignals), 10)
-        self.assertEqual(len(block.channel_indexes[-1].units), 4)
-        self.assertEqual(len(block.channel_indexes[-1].units),
-                         len(block.segments[0].spiketrains))
+        #self.assertEqual(len(block.channel_indexes[-1].units), 4)
+        #self.assertEqual(len(block.channel_indexes[-1].units),
+        #                 len(block.segments[0].spiketrains))
 
         anasig = block.segments[0].analogsignals[0]
         self.assertIsNotNone(anasig.file_origin)
@@ -134,7 +134,7 @@ class CommonTests(BaseTestIO, unittest.TestCase):
         #     reader2 = BlackrockIO(filename=filename, nev_override='nonexistent')
 
         # Load data to maximum extent, one None is not given as list
-        block = reader.read_block(load_waveforms=False)
+        block = reader.read_block(load_waveforms=False, signal_group_mode='split-all')
         lena = len(block.segments[0].analogsignals[0])
         numspa = len(block.segments[0].spiketrains[0])
 
@@ -143,8 +143,11 @@ class CommonTests(BaseTestIO, unittest.TestCase):
         too_large_tstop = block.segments[0].analogsignals[0].t_stop + 1 * pq.s
         buggy_slice = (-100 * pq.ms, too_large_tstop)
 
-        # This is valid in read_segment because seg_index is specified
-        seg = reader.read_segment(seg_index=0, time_slice=buggy_slice)
+        # This is valid in read_segment
+        seg = reader.read_segment(seg_index=0, time_slice=buggy_slice, strict_slicing=False)
+        # this raise error
+        with self.assertRaises(AssertionError):
+            seg = reader.read_segment(seg_index=0, time_slice=buggy_slice, strict_slicing=True)
 
         lenb = len(seg.analogsignals[0])
         numspb = len(seg.spiketrains[0])
@@ -159,13 +162,13 @@ class CommonTests(BaseTestIO, unittest.TestCase):
 
         # test 4 Units
         block = reader.read_block(load_waveforms=True,
-                                  units_group_mode='all-in-one')
+                                  signal_group_mode='split-all',
+                                  )#units_group_mode='all-in-one')
 
         self.assertEqual(len(block.segments[0].analogsignals), 96)
-        self.assertEqual(len(block.channel_indexes[-1].units), 218)
-        self.assertEqual(len(block.channel_indexes[-1].units),
-                         len(block.segments[0].spiketrains))
-
+        #self.assertEqual(len(block.channel_indexes[-1].units), 218)
+        #self.assertEqual(len(block.channel_indexes[-1].units),
+        #                 len(block.segments[0].spiketrains))
         anasig = block.segments[0].analogsignals[0]
         self.assertIsNotNone(anasig.file_origin)
 
@@ -181,12 +184,42 @@ class CommonTests(BaseTestIO, unittest.TestCase):
         block = reader.read_block(load_waveforms=False)
         sampling_rates = np.unique(
             [a.sampling_rate.rescale('Hz') for a in block.filter(objects='AnalogSignal')])
-        self.assertEqual(len(sampling_rates), len(reader._selected_nsx))
+        self.assertEqual(len(sampling_rates), 2)
 
         segment = reader.read_segment()
         sampling_rates = np.unique(
             [a.sampling_rate.rescale('Hz') for a in segment.filter(objects='AnalogSignal')])
-        self.assertEqual(len(sampling_rates), len(reader._selected_nsx))
+        self.assertEqual(len(sampling_rates), 2)
+
+        # load only ns5
+        reader = BlackrockIO(filename=filename, nsx_to_load=5)
+        seg = reader.read_segment()
+        self.assertEqual(len(seg.analogsignals), 1)
+        self.assertEqual(seg.analogsignals[0].shape, (109224, 96))
+
+        # load only ns2
+        reader = BlackrockIO(filename=filename, nsx_to_load=2)
+        seg = reader.read_segment()
+        self.assertEqual(len(seg.analogsignals), 1)
+        self.assertEqual(seg.analogsignals[0].shape, (3640, 6))
+
+        # load only ns2
+        reader = BlackrockIO(filename=filename, nsx_to_load=[2])
+        seg = reader.read_segment()
+        self.assertEqual(len(seg.analogsignals), 1)
+
+        # load ns2 + ns5
+        reader = BlackrockIO(filename=filename, nsx_to_load=[2, 5])
+        seg = reader.read_segment()
+        self.assertEqual(len(seg.analogsignals), 2)
+        self.assertEqual(seg.analogsignals[0].shape, (3640, 6))
+        self.assertEqual(seg.analogsignals[1].shape, (109224, 96))
+
+        # load only ns5
+        reader = BlackrockIO(filename=filename, nsx_to_load='max')
+        seg = reader.read_segment()
+        self.assertEqual(len(seg.analogsignals), 1)
+        self.assertEqual(seg.analogsignals[0].shape, (109224, 96))
 
     @unittest.skipUnless(HAVE_SCIPY, "requires scipy")
     def test_compare_blackrockio_with_matlabloader_v21(self):
@@ -224,20 +257,20 @@ class CommonTests(BaseTestIO, unittest.TestCase):
             session = BlackrockIO(
                 dirname,
                 verbose=False, **param[1])
-            block = session.read_block(load_waveforms=True)
+            block = session.read_block(load_waveforms=True, signal_group_mode='split-all')
             # Check if analog data are equal
-            self.assertGreater(len(block.channel_indexes), 0)
-            for i, chidx in enumerate(block.channel_indexes):
+            self.assertGreater(len(block.groups), 0)
+            for i, chidx_grp in enumerate(block.channel_indexes):
                 # Break for ChannelIndexes for Units that don't contain any Analogsignals
-                if len(chidx.analogsignals) == 0 and len(chidx.units) >= 1:
+                if len(chidx_grp.analogsignals) == 0 and len(chidx_grp.spiketrains) >= 1:
                     break
-                # Should only have one AnalogSignal per ChannelIndex
-                self.assertEqual(len(chidx.analogsignals), 1)
+                # Should only have one AnalogSignal per ChannelIndex-representing Group
+                self.assertEqual(len(chidx_grp.analogsignals), 1)
 
                 # Find out channel_id in order to compare correctly
-                idx = chidx.analogsignals[0].annotations['channel_id']
+                idx = chidx_grp.analogsignals[0].annotations['channel_id']
                 # Get data of AnalogSignal without pq.units
-                anasig = np.squeeze(chidx.analogsignals[0].base[:].magnitude)
+                anasig = np.squeeze(chidx_grp.analogsignals[0].base[:].magnitude)
                 # Test for equality of first nonzero values of AnalogSignal
                 #                                   and matlab file contents
                 # If not equal test if hardcoded gain is responsible for this
@@ -245,7 +278,7 @@ class CommonTests(BaseTestIO, unittest.TestCase):
                 j = 0
                 while anasig[j] == 0:
                     j += 1
-                if lfp_ml[i, j] != np.squeeze(chidx.analogsignals[0].base[j].magnitude):
+                if lfp_ml[i, j] != np.squeeze(chidx_grp.analogsignals[0].base[j].magnitude):
                     anasig = anasig / 152.592547
                     anasig = np.round(anasig).astype(int)
 
@@ -275,7 +308,7 @@ class CommonTests(BaseTestIO, unittest.TestCase):
                 matlab_spikes = ts_ml[np.nonzero(
                     np.logical_and(elec_ml == channelid, unit_ml == unitid))]
                 # Going sure that unit is really seconds and not 1/30000 seconds
-                if (not st_i.units == pq.CompoundUnit("1.0/{0} * s".format(30000))) and \
+                if (not st_i.units == pq.CompoundUnit("1.0/{} * s".format(30000))) and \
                         st_i.units == pq.s:
                     st_i = np.round(st_i.base * 30000).astype(int)
                 assert_equal(st_i, matlab_spikes)
