@@ -1,13 +1,8 @@
-# -*- coding: utf-8 -*-
 """
 Tests of neo.io.brainwaredamio
 """
 
-# needed for python 3 compatibility
-from __future__ import absolute_import, division, print_function
-
 import os.path
-import sys
 
 import unittest
 
@@ -15,15 +10,13 @@ import numpy as np
 import quantities as pq
 
 from neo.core import (AnalogSignal, Block,
-                      ChannelIndex, Segment)
+                      Group, Segment)
 from neo.io import BrainwareDamIO
 from neo.test.iotest.common_io_test import BaseTestIO
 from neo.test.tools import (assert_same_sub_schema,
                             assert_neo_object_is_compliant)
 from neo.test.iotest.tools import create_generic_reader
 
-PY_VER = sys.version_info[0]
-
 
 def proc_dam(filename):
     '''Load an dam file that has already been processed by the official matlab
@@ -55,12 +48,9 @@ def proc_dam(filename):
 
     block = Block(file_origin=filename)
 
-    chx = ChannelIndex(file_origin=filename,
-                       index=np.array([0]),
-                       channel_ids=np.array([1]),
-                       channel_names=np.array(['Chan1'], dtype='S'))
+    gr = Group(file_origin=filename)
 
-    block.channel_indexes.append(chx)
+    block.groups.append(gr)
 
     params = [res['params'][0, 0].flatten() for res in damfile['stim']]
     values = [res['values'][0, 0].flatten() for res in damfile['stim']]
@@ -79,6 +69,8 @@ def proc_dam(filename):
                           **stim)
         segment.analogsignals = [sig]
         block.segments.append(segment)
+        gr.analogsignals.append(sig)
+        sig.group = gr
 
     block.create_many_to_one_relationship()
 
@@ -108,10 +100,10 @@ class BrainwareDamIOTestCase(BaseTestIO, unittest.TestCase):
                         'random_500ms_12rep_noclust_part_ch2',
                         'sequence_500ms_5rep_ch2']
 
-    # add the appropriate suffix depending on the python version
+    # add the suffix
     for i, fname in enumerate(files_to_compare):
         if fname:
-            files_to_compare[i] += '_dam_py%s.npz' % PY_VER
+            files_to_compare[i] += '_dam_py3.npz'
 
     # Will fetch from g-node if they don't already exist locally
     # How does it know to do this before any of the other tests?
@@ -126,7 +118,7 @@ class BrainwareDamIOTestCase(BaseTestIO, unittest.TestCase):
             obj_single = obj_reader_single()
 
             try:
-                assert_same_sub_schema(obj_base, obj_single)
+                assert_same_sub_schema(obj_base, [obj_single])
             except BaseException as exc:
                 exc.args += ('from ' + os.path.basename(path),)
                 raise
