@@ -1,28 +1,21 @@
-# -*- coding: utf-8 -*-
 """
 Tests of neo.io.brainwaref32io
 """
 
-# needed for python 3 compatibility
-from __future__ import absolute_import, division, print_function
-
 import os.path
-import sys
 
 import unittest
 
 import numpy as np
 import quantities as pq
 
-from neo.core import Block, ChannelIndex, Segment, SpikeTrain, Unit
+from neo.core import Block, Segment, SpikeTrain, Group
 from neo.io import BrainwareF32IO
 from neo.test.iotest.common_io_test import BaseTestIO
 from neo.test.tools import (assert_same_sub_schema,
                             assert_neo_object_is_compliant)
 from neo.test.iotest.tools import create_generic_reader
 
-PY_VER = sys.version_info[0]
-
 
 def proc_f32(filename):
     '''Load an f32 file that has already been processed by the official matlab
@@ -48,19 +41,13 @@ def proc_f32(filename):
 
     # create the objects to store other objects
     block = Block(file_origin=filenameorig)
-    chx = ChannelIndex(file_origin=filenameorig,
-                       index=np.array([], dtype=np.int),
-                       channel_names=np.array([], dtype='S'))
-    unit = Unit(file_origin=filenameorig)
-
-    # load objects into their containers
-    block.channel_indexes.append(chx)
-    chx.units.append(unit)
+    gr = Group(file_origin=filenameorig)
+    block.groups.append(gr)
 
     try:
         with np.load(filename, allow_pickle=True) as f32obj:
             f32file = list(f32obj.items())[0][1].flatten()
-    except IOError as exc:
+    except OSError as exc:
         if 'as a pickle' in exc.message:
             block.create_many_to_one_relationship()
             return block
@@ -88,7 +75,7 @@ def proc_f32(filename):
 
             segment = Segment(file_origin=filenameorig, **params)
             segment.spiketrains = [train]
-            unit.spiketrains.append(train)
+            gr.spiketrains.append(train)
             block.segments.append(segment)
 
     block.create_many_to_one_relationship()
@@ -113,8 +100,8 @@ class BrainwareF32IOTestCase(BaseTestIO, unittest.TestCase):
                      'random_500ms_12rep_noclust_part_ch2.f32',
                      'sequence_500ms_5rep_ch2.f32']
 
-    # add the appropriate suffix depending on the python version
-    suffix = '_f32_py%s.npz' % PY_VER
+    # add the appropriate suffix
+    suffix = '_f32_py3.npz'
     files_to_download = files_to_test[:]
 
     # add the reference files to the list of files to download
@@ -136,7 +123,7 @@ class BrainwareF32IOTestCase(BaseTestIO, unittest.TestCase):
             obj_single = obj_reader_single()
 
             try:
-                assert_same_sub_schema(obj_base, obj_single)
+                assert_same_sub_schema(obj_base, [obj_single])
             except BaseException as exc:
                 exc.args += ('from ' + os.path.basename(path),)
                 raise
