@@ -1,12 +1,7 @@
-# -*- coding: utf-8 -*-
 """
 Tests of neo.io.kwikio
 """
 
-# needed for python 3 compatibility
-from __future__ import division
-
-import sys
 import unittest
 
 try:
