@@ -1,13 +1,7 @@
-# -*- coding: utf-8 -*-
 """
 Tests of neo.io.neomatlabio
 """
 
-# needed for python 3 compatibility
-from __future__ import absolute_import, division
-
-import sys
-
 import unittest
 
 from neo.io import MicromedIO
