@@ -1,11 +1,7 @@
-# -*- coding: utf-8 -*-
 """
 Tests of io.rawbinarysignal
 """
 
-# needed for python 3 compatibility
-from __future__ import absolute_import, division
-
 import unittest
 
 from neo.io import RawBinarySignalIO
